package eu.etaxonomy.taxeditor;

import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;
import org.eclipse.ui.application.IWorkbenchWindowConfigurer;
import org.eclipse.ui.application.WorkbenchWindowAdvisor;

import eu.etaxonomy.taxeditor.datasource.CdmDataSourceRepository;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.dialog.LoginDialog;
import eu.etaxonomy.taxeditor.update.P2Util;
import eu.etaxonomy.taxeditor.util.ApplicationUtil;

/**
 * <p>ApplicationWorkbenchWindowAdvisor class.</p>
 *
 * @author n.hoffmann
 * @version $Id: $
 */
public class ApplicationWorkbenchWindowAdvisor extends WorkbenchWindowAdvisor {

	/**
	 * <p>Constructor for ApplicationWorkbenchWindowAdvisor.</p>
	 *
	 * @param configurer a {@link org.eclipse.ui.application.IWorkbenchWindowConfigurer} object.
	 */
	public ApplicationWorkbenchWindowAdvisor(IWorkbenchWindowConfigurer configurer) {
		super(configurer);
	}

	/** {@inheritDoc} */
	@Override
    public ActionBarAdvisor createActionBarAdvisor(
			IActionBarConfigurer configurer) {
		return new ApplicationActionBarAdvisor(configurer);
	}

	/**
	 * <p>preWindowOpen</p>
	 */
	@Override
    public void preWindowOpen() {
		IWorkbenchWindowConfigurer configurer = getWindowConfigurer();
		configurer.setInitialSize(new Point(963, 637));
		configurer.setShowCoolBar(true);
		configurer.setShowStatusLine(true);
		configurer.setShowPerspectiveBar(true);
		configurer.setTitle(ApplicationUtil.getTitle());
		configurer.setShowProgressIndicator(true);

		CdmDataSourceRepository.createDefaultH2DataSource();
	}

	/**
	 * <p>postWindowOpen</p>
	 */
	@Override
    public void postWindowOpen() {
		PreferencesUtil.checkNomenclaturalCode();



		if(PreferencesUtil.shouldConnectAtStartUp()) {
            CdmStore.connect();
        }

		P2Util.setP2UpdateRepositories();

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.application.WorkbenchWindowAdvisor#postWindowClose()
	 */
	@Override
    public void postWindowClose() {
        P2Util.saveP2RepositoryPreferences();
    }

	private int authenticate(){
		Shell shell = TaxonomicEditorPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell();

		LoginDialog loginDialog = new LoginDialog(shell);
		return loginDialog.open();
	}
}

